/***************************************************************************\
*****************************************************************************
**                                                                         **
** source file:    file_io.c                                               **
** last revision:  17/12/2003                                              **
**                                                                         **
*****************************************************************************
*****************************************************************************
**                                                                         **
** This program is free software; you can redistribute it and/or modify it **
** under the terms of the GNU General Public License as published by the   **
** Free Software Foundation; either version 2, or (at your option) any     **
** later version.                                                          **
**                                                                         **
** This program is distributed in the hope that it will be useful,         **
** but WITHOUT ANY WARRANTY; without even the implied warranty of          **
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           **
** GNU General Public License for more details.                            **
**                                                                         **
** You should have received a copy of the GNU General Public License       **
** along with this program; if not, write to the Free Software             **
** Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.                     **
**                                                                         **
*****************************************************************************
\***************************************************************************/

//---------------------------------------------------------------------------------------
//|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
//---------------------------------------------------------------------------------------

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include "error.h"
#include "file_io.h"

//---------------------------------------------------------------------------------------
//|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
//---------------------------------------------------------------------------------------

int open_file(const char *name, const int flags)
{
  int new_file;

  if ((new_file = open(name, flags, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)) < 0)
    {
      error("open", NULL, errno);
      return -1;
    }

  return new_file;
}


//---------------------------------------------------------------------------------------
//|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
//---------------------------------------------------------------------------------------

size_t read_from_file(const int file, void *buffer, size_t size)
{
  int tmp;

  if ((tmp = read(file, buffer, size)) < 0)
    {
      error("read_from_file", NULL, errno);
      return -1;
    }

  return tmp;
}

//---------------------------------------------------------------------------------------
//|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
//---------------------------------------------------------------------------------------

size_t write_to_file(const int file, void *buffer, size_t size)
{
  int tmp;

  if ((tmp = write(file, buffer, size)) < 0)
    {
      error("write_to_file", NULL, errno);
      return -1;
    }

  return tmp;
}

//---------------------------------------------------------------------------------------
//|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
//---------------------------------------------------------------------------------------

int close_file(const int file)
{
  if ((close(file)) < 0)
    {
      error("close", NULL, errno);
      return -1;
    }

  return 0;
}

//---------------------------------------------------------------------------------------
//|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
//---------------------------------------------------------------------------------------
