/***************************************************************************\
*****************************************************************************
**                                                                         **
** SFT (Simple File Transfer)                                              **
** Version 0.4.14     28/03/2004                                           **
** Started the 11th december 2002                                          **
** By Nicolas BENOIT, nbenoit@tuxfamily.org                                **
** http://nbenoit.tuxfamily.org                                            **
** Copyright (C) 2002, 2003, 2004 Nicolas BENOIT                           **
**                                                                         **
*****************************************************************************
*****************************************************************************
**                                                                         **
** source file:    main.c                                                  **
** last revision:  28/03/2004                                              **
**                                                                         **
*****************************************************************************
*****************************************************************************
**                                                                         **
** This program is free software; you can redistribute it and/or modify it **
** under the terms of the GNU General Public License as published by the   **
** Free Software Foundation; either version 2, or (at your option) any     **
** later version.                                                          **
**                                                                         **
** This program is distributed in the hope that it will be useful,         **
** but WITHOUT ANY WARRANTY; without even the implied warranty of          **
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           **
** GNU General Public License for more details.                            **
**                                                                         **
** You should have received a copy of the GNU General Public License       **
** along with this program; if not, write to the Free Software             **
** Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.                     **
**                                                                         **
*****************************************************************************
\***************************************************************************/

//---------------------------------------------------------------------------------------
//|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
//---------------------------------------------------------------------------------------

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "common.h"
#include "memory.h"
#include "init.h"
#include "server.h"
#include "client.h"

//---------------------------------------------------------------------------------------
//|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
//---------------------------------------------------------------------------------------

int main(int argc, char *argv[])
{
  struct options_t prog_options;  // The Options
  MEM_MANAGER *mem_manager;       // The Main Memory Manager
  MEM_MANAGER *opt_mem_manager;   // The Memory Manager For The Options

  /** We Initialize The Main Memory Manager **/
  if ((mem_manager = mem_create()) == NULL)
    return -1;

  /** We Initialize The Memory Manager For The Options**/
  if ((opt_mem_manager = mem_create()) == NULL)
    return -1;

  /** We Get The Options **/
  if ((init_arg(argc, argv, &prog_options, opt_mem_manager)) != 0)
    return -1;

  /** We Launch Server Or Client Mode According To The Options **/
  switch (prog_options.mode)
    {
    case 's':
      {
        if (prog_options.keep_running)
          {
            int i = 1;
            char *working_directory = NULL;

            // We Save The Working Directory (just in case)
            if (prog_options.dir != NULL)
              {
                working_directory = malloc(1+strlen(prog_options.dir));
                strcpy(working_directory, prog_options.dir);
              }

            fprintf(stderr, "\n####################################################\n");
            fprintf(stderr, "\nRunning %s in server mode (session 1) ...\n\n", PROGRAM_NAME);

            // We Launch The Server For The Eternity
            while (!(start_server(prog_options.port, &prog_options.limit, mem_manager)))
              {
                ++i;                       // We Increment The Session Counter
                mem_free_all(mem_manager); // We Free The Previously Allocated Memory
                fprintf(stderr, "####################################################\n");
                fprintf(stderr, "\nRunning %s in server mode (session %d) ...\n\n", PROGRAM_NAME, i);
              }

            if (working_directory != NULL) free(working_directory);
            mem_destroy(mem_manager);
            mem_destroy(opt_mem_manager);
          }
        else
          {
            fprintf(stderr, "\nRunning %s in server mode...\n\n", PROGRAM_NAME);
            start_server(prog_options.port, &prog_options.limit, mem_manager);
            mem_destroy(mem_manager);
            mem_destroy(opt_mem_manager);
          }

        break;
      }

    case 'c':
      {
        fprintf(stderr, "\nRunning %s in client mode...\n\n", PROGRAM_NAME);
        start_client(prog_options.port, prog_options.address, prog_options.nbr_files, prog_options.file, mem_manager);
        mem_destroy(mem_manager);
        mem_destroy(opt_mem_manager);
        break;
      }
    }

  return EXIT_SUCCESS;
}


//---------------------------------------------------------------------------------------
//|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
//---------------------------------------------------------------------------------------
