/* -*- Mode: C; tab-width: 2; indent-tabs-mode: t; c-basic-offset: 2 -*-
 * File: dns.c
 *
 * Created: Mon May 19 10:18:58 2003
 *
 * $Id: dns.c,v 1.1 2003/12/13 20:59:15 mhassler Exp $
 */

#include <stdio.h>
#include <endian.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <malloc.h>
#include <string.h>
#include <arpa/inet.h>

#include "dns.h"

#ifdef DMALLOC
#include <dmalloc.h>
#endif /* DMALLOC */

unsigned short globalid = 1;

int dns_query(char *name, int size)
{
	struct dns_header hdr;
	struct sockaddr_in sin;
	int len;
	int sok, ret;
	char *buf, *ptr, *pos;

	sok = socket(AF_INET, SOCK_DGRAM, 0);
	if(sok < 0)
		return -1;

	sin.sin_family = AF_INET;
	sin.sin_port = htons(53);
	sin.sin_addr.s_addr = htonl(NAMESERVER);
	len = sizeof(struct sockaddr_in);
	if(connect(sok, (struct sockaddr*)&sin, len) < 0)
		return -1;

	hdr.id = htons(globalid++);
	hdr.qr = 0; // query
	hdr.opcode = 0;
	hdr.aa = 0;
	hdr.tc = 0;
	hdr.rd = 1; // recursive query
	hdr.ra = 0;
	hdr.z = 0;
	hdr.rcode = 0;
	hdr.qdcount = htons(1); // 1 query
	hdr.ancount = htons(0);
	hdr.nscount = htons(0);
	hdr.arcount = htons(0);

	buf = malloc(sizeof(struct dns_header) + size + 10);
	if(!buf)
		return -1;

	memcpy(buf, &hdr, sizeof(struct dns_header));
	ptr = &buf[sizeof(struct dns_header)];

	while((pos = memchr(name, '.', size)))
	{
		len = pos - name;
		*ptr++ = len;
		memcpy(ptr, name, len);
		ptr += len;
		name = pos+1;
		size -= len+1;
	}
	len = size;
	*ptr++ = len;
	memcpy(ptr, name, len);
	ptr += len;
	*ptr++ = 0;

	*ptr++ = 0;
	*ptr++ = 1;
	*ptr++ = 0;
	*ptr++ = 1;

	ret = send(sok, buf, ptr-buf, 0);
	free(buf);

	return ret < 0?-1:sok;
}

unsigned long dns_read(int sok)
{
	char buf[4096], *ptr;
	struct dns_header *hdr = (void*)buf;
	int i, q, len, type;

	if(recv(sok, buf, sizeof(buf), 0) < 0)
		return 0;

	if(!ntohs(hdr->qr) || ntohs(hdr->rcode) || !ntohs(hdr->ancount))
		return 0;

	ptr = &buf[sizeof(struct dns_header)];

	q = ntohs(hdr->qdcount);
	for(i = 0; i < q; i++)
	{
		dns_parse_record(buf, &ptr);
		ptr += 2; // type
		ptr += 2; // class
	}

	q = ntohs(hdr->ancount);
	for(i = 0; i < q; i++)
	{
		dns_parse_record(buf, &ptr);
		type = ntohs(*(unsigned short*)ptr);
		ptr += 2; // type
		ptr += 2; // class
		ptr += 4; // TTL
		len = ntohs(*(unsigned short*)ptr);
		ptr += 2; // len
		if(type == 1 && len == 4)
		{
//printf("IP address: '%s'\n", inet_ntoa(*(struct in_addr*)ptr));
			return ntohl(*(unsigned long*)ptr);
		}
		ptr += len;
	}
	return 0;
}

char *dns_parse_label(char *buf, char *ptr, char **name, int *total)
{
	int len, off;
	unsigned short ref;

	while(*ptr)
	{
		if((*ptr & 0xC0) == 0xC0)
		{
			// pointer
			ref = ntohs(*(unsigned short*)ptr);
			off = ref & ~0xC000;
			dns_parse_label(buf, &buf[off], name, total);
			ptr += 2;
			return ptr;
		}
		else
		{
			// size + label
			len = *ptr;
			*name = realloc(*name, (*total)+len+2);
			memcpy((*name)+(*total), ptr+1, len);
			(*name)[(*total)+len] = '.';
			(*name)[(*total)+len+1] = 0;
			*total += len+1;
			ptr += len+1;
		}
	}
	ptr++;
	return ptr;
}

int dns_parse_record(char *buf, char **ptr)
{
	int total = 0;
	// initialize with empty string
	char *name = malloc(1);
	*name = 0;

	*ptr = dns_parse_label(buf, *ptr, &name, &total);
//printf("The name is: '%s'\n", name);
	free(name);
	return 0;
}
