/* -- updatedd: noip.c --
 *
 * Copyright (C) 2002, 2003, 2004 Philipp Benner
 *
 * This file is part of UpdateDD - http://updatedd.philipp-benner.de.
 *
 * UpdateDD is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * any later version.
 *
 * UpdateDD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with UpdateDD; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <config.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <ctype.h>
#include <unistd.h>
#include <getopt.h>

#include <get_connection.h>
#include <libexception_handle.h>
#include <ret_codes.h>
#include <version.h>

#include "noip.h"
#include "cachefile.h"

char login[BUFSIZE];

static void
print_usage(char *pname, FILE *fp)
{
	(void)fprintf(fp,
		      "\nUsage: %s [...] %s -- [OPTION]... [USERNAME:PASSWORD] HOSTNAME\n\n",
		      pname, COLORED("noip"));
	(void)fprintf(fp,
		      "For security reasons use the environment variable LOGIN instead of\n"
		      "passing the login information directly.\n\n"
		      "Options:\n"
		      "   -4    --ipv4 <address>        ip address version 4\n"
                      "   -g    --group                 use group instead of HOSTNAME\n"
		      "         --help                  print help and exit\n"
		      "         --version               display version information and exit\n\n"
		      "Report bugs to <"EMAIL">.\n\n");

	return;
}

static void
print_version(FILE *fp)
{

	(void)fprintf(fp,
		      "\n" PNAME " plugin for no-ip.com version " VERSION ",\n"
		      "Copyright (C) 2004 Philipp Benner.\n"
		      HOMEPAGE "\n\n"

		      "This is free software, and you are welcome to redistribute it\n"
		      "under certain conditions; see the source for copying conditions.\n"
		      "There is NO warranty; not even for MERCHANTABILITY or FITNESS\n"
		      "FOR A PARTICULAR PURPOSE.\n\n");

	return;

}

int
dyndns(int argc, char *argv[])
{

	struct arguments args;
	const char *ptr;
	int s, ret = RET_WRONG_USAGE;
	char filez[32];

        printf("noip plugin started ...\n");
#ifdef NSP
        memset((void *)&args, 0xFF, sizeof(struct arguments));
#else
        (void)memset(&args, 0, sizeof(struct arguments));
#endif
	
	if(get_flags(&args, argc, argv) != RET_OK) {
		return RET_WRONG_USAGE;
	}

        sprintf(filez, "/var/tmp/noip.%s", args.iface);
        if(!check_ipcache(args.ipv4, filez) && !strcmp(args.force, "no"))
        {
            printf("noip not update.nochg\n");
            return RET_OK;
        }
 
        if(!strcmp(args.force, "yes"))
            printf("noip force update...\n");
 
        s = get_connection(DYNDNSHOST, args.iface, PORT, &ptr);
	if(s == -1) {
		printf("%s: %s\n", ptr, DYNDNSHOST);
		exit(EXIT_FAILURE);
	} else {
		ret = update_dyndns(s, &args);
		if(ret == RET_OK) {
                        ret = check_server_msg(s, args.hostname);
		}
		(void)close(s);
                if(ret == RET_OK)
                        save_ipcache(args.ipv4, filez);
	}

	return ret;

}

static int
get_flags(struct arguments *args, int argc, char *argv[])
{

	int c;
	char **ptr = &args->hostname;
	
	args->force = "no";
	args->group = "off";

        if(argc == 1)
        {
                print_usage(argv[ARGV_PNAME], stdout);
                printf("Try `updatedd --help' for more information\n");
                exit(EXIT_FAILURE);
        }
	
	for(;;) {

		int option_index = 0;
		static struct option long_options[] = {
			{ "ipv4",		1, 0, '4' },
			{ "help",		0, 0, 'h' },
			{ "force",              0, 0, 'f' },
 			{ "group",		1, 0, 'g' },
			{ "version",		0, 0, 'v' },
                        { "interface",          1, 0, 'i' },
                        { "user",               1, 0, 'u' },
                        { "pass",               1, 0, 'p' },
			{ NULL,			0, 0, 0   }
		};

		c = getopt_long(argc, argv, "4:i:f:g:u:p:",
				long_options, &option_index);

		if(c == -1) break;

		switch(c) {
		case '4':
			args->ipv4 = optarg;
			break;
 		case 'g':
 			args->group = optarg; /* use hostname as groupname */
 			break;
		case 'h':
			print_usage(argv[ARGV_PNAME], stdout);
			exit(EXIT_SUCCESS);
		case 'v':
			print_version(stdout);
			exit(EXIT_SUCCESS);
                case 'f':
                        args->force = "yes";
                        break;
                case 'i':
                        args->iface = optarg;
                        break;
                case 'u':
                        strcpy(login,optarg);
                        strcat(login,":");
                        break;
                case 'p':
                        strcat(login,optarg);

		}
	}

        if(strcmp(login,"") == 0)
        {
                args->login = getenv("LOGIN");
                if(args->login == NULL) {
                        printf("environment variable LOGIN is empty\n");
                        return RET_WRONG_USAGE;
                }
        }

	*ptr = argv[ARGV_HOSTNAME];

	return RET_OK;
}

#define BUFLEN		4096
#define BUFFREE(name)	BUFLEN - strlen(name)

static int
update_dyndns(int s, struct arguments *args)
{

	char message[BUFLEN];
	int i;
	int len = strlen(login);
        char *pass = NULL;

	for(i = 0; i < len; i++) {
		if(login[i] == ':') {
			login[i] = '\0';
                        pass = login+i+1;
			break;
		}
	}
	if(pass == NULL) {
		printf("password is missing\n");
		return RET_WRONG_USAGE;
	}

	(void)snprintf(message, BUFLEN,
		       "GET /update.php?username=%s&pass=%s",
		       login, pass );

	if(args->ipv4) {
		(void)strncat(message, "&ip=", BUFFREE(message));
		(void)strncat(message, args->ipv4, BUFFREE(message));
	}

	if(!strcmp(args->group, "on") && args->hostname)
	{    
		printf("using group....\n");
		(void)strncat(message, "&groupname=", BUFFREE(message));
		(void)strncat(message, args->hostname, BUFFREE(message));
	} else if(args->hostname) {
		printf("using hostname....\n");
		(void)strncat(message, "&host=", BUFFREE(message));
		(void)strncat(message, args->hostname, BUFFREE(message));
	} else {
		printf("wrong usage\n");
		return RET_WRONG_USAGE;
	}

	{
		char buffer[1024];

		(void)snprintf(buffer, 1024,
                               " HTTP/1.1\r\n"
                               "Host: %s\r\n"
                               "User-Agent: %s %s - %s\r\n"
                               "Connection: close\r\n"
                               "Pragma: no-cache\r\n\r\n",
                               DYNDNSHOST, PNAME, VERSION, HOMEPAGE);
		(void)strncat(message, buffer, BUFFREE(message));
	}
	print_debug("\n\nMessage:"
		    "\n--------------------------------------\n"
		    "%s--------------------------------------\n\n",
                    message);
	
	if(write(s, message, strlen(message)) == -1) {
		printf("write() failed\n");
		return RET_WARNING;
	}

	return RET_OK;

}

static int
check_server_msg(int s, char *hostname)
{

	int n, ret_code;
	char server_msg[BUFSIZE], *ptr;

	printf("Inside check_server_msg\n");
	
	(void)memset((void *)server_msg, 0, sizeof(server_msg));
	if(read(s, server_msg, sizeof(server_msg) - 1) < 0) {
		printf("read() failed\n");
		return RET_WARNING;
        }

	print_debug("\n\nServer message:"
		    "\n--------------------------------------\n"
		    "%s--------------------------------------\n\n",
		    server_msg);

	if(strstr(server_msg, "HTTP/1.1 200 OK") ||
           strstr(server_msg, "HTTP/1.0 200 OK") ) {

		ptr = strstr(server_msg, "status=");
		ret_code = atoi((ptr+7));
		for(n=0; return_codes[n].message != NULL; n++) {
			if(ret_code == return_codes[n].code) {
				printf("%s: %s\n", hostname, return_codes[n].message);
				if(return_codes[n].error == 1) {
					return RET_ERROR;
				} else {
					break;
				}
			}
		}
	} else {
		printf("no-ip.com: Internal Server Error.error\n");
		return RET_ERROR;
	}

	return RET_OK;
}
